program difusao_do_calor

! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
!

! Mdulos

use constantes
use calculos
use inflow_outflow
use solucao
use estabilizacao			
use resultados
use indices_fisicos

implicit none

integer:: n				! nmero de pontos em relao ao tempo
integer:: j				! contador para o tempo
integer:: i 			! contador do espao
integer:: m				! nmero de volumes


double precision, allocatable, dimension (:):: T		! temperatura da gua
double precision, allocatable, dimension (:):: Tn		! temperatura da gua em j+1 sem estabilizao do perfil
double precision, allocatable,  dimension(:):: Tz		! Temperatura da gua com estabilizao
double precision, allocatable,  dimension(:):: H		! Profundidade do reservatrio
double precision, allocatable,  dimension(:):: Hn		! Profundidade do reservatrio
double precision, allocatable, dimension (:):: nivel
double precision, allocatable,  dimension(:):: zh		! elevao de cada centro de volume

double precision, allocatable, dimension (:):: UR		! umidade relativa
double precision, allocatable, dimension (:):: Ta		! temperatura da gua
double precision, allocatable, dimension (:):: U0		! velocidade do vento, dado bruto
double precision, allocatable, dimension (:):: U		! velocidade do vento
double precision, allocatable, dimension (:):: qs		! radiao de ondas curtas
double precision, allocatable, dimension (:):: qn		! Fluxo de energia na superfcie
integer, allocatable, dimension (:):: dia
integer, allocatable, dimension (:):: mes
integer, allocatable, dimension (:):: ano

double precision, allocatable, dimension (:):: cp			! calor especfico	
double precision, allocatable, dimension (:):: rho			! massa especfica

double precision, allocatable, dimension (:):: Kh				! coeficiente de difuso turbulenta	
double precision, allocatable, dimension (:):: z				! Varivel auxiliar
double precision, allocatable, dimension (:):: w				! Varivel auxiliar
double precision, allocatable, dimension (:):: N2				! Frequncia de Brunt-Vaisala
double precision, allocatable, dimension (:):: Ri				! Nmero de Richardson
double precision, allocatable, dimension(:)::  Ar
double precision, allocatable, dimension(:):: a 		! Coeficiente ap do sistema de coeficentes
double precision, allocatable, dimension(:):: b 		! Coeficiente aw do sistema de coeficentes
double precision, allocatable, dimension(:):: c 		! Coeficiente ae do sistema de coeficentes
double precision, allocatable, dimension(:):: d 		! Coeficiente b do sistema de coeficentes


double precision, allocatable, dimension(:):: E
double precision, allocatable, dimension(:):: BE
double precision, allocatable, dimension(:):: BE1


double precision, allocatable, dimension (:,:):: Taux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, allocatable, dimension (:,:):: zaux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, allocatable, dimension (:,:):: Khaux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, allocatable, dimension (:,:):: Riaux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, allocatable, dimension (:,:):: N2aux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, allocatable, dimension (:,:):: rhoaux			! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
integer, dimension(np):: m_aux									! auxiliar para calcular os indices fisicos
integer, allocatable, dimension(:):: maux
integer:: aux_media

double precision, dimension (m_max,np):: Ts	    ! temperatura da gua com escala de tempo diferente da calculada, e maior que dt


double precision, allocatable, dimension (:):: Haux
double precision, allocatable, dimension (:):: Q_in
double precision, allocatable, dimension (:):: Q_out

real:: time1, time2, time3

double precision, dimension (np):: Us			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: qss			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: URs			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: Tas			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (m_max,np):: rhos
integer, dimension(np):: aux
integer, dimension (np)::aux_ep_depth
integer, dimension (np)::aux_hipo_depth
double precision, dimension (np):: hipolimnio  						 ! profundidade do hipolimnio  
double precision, dimension (np):: epilimnio  						 ! profundidade do epilimnio
double precision, dimension (np):: rho_hipo      					 ! densidade do hipolimnio
double precision, dimension (np):: rho_epi     						 ! densidade do epimlimnio
double precision, allocatable, dimension (:):: Kh_vazao				 ! difuso turbulenta
double precision, allocatable, dimension (:):: q				 	 ! radiao solar -  lei de Beer, absorvida com a profundidade do lago
double precision, allocatable, dimension (:):: Qin
double precision, allocatable, dimension (:):: Qout
double precision, allocatable, dimension (:):: Tam					 ! temperatura mdia do ar		
double precision, allocatable, dimension (:):: Tin				 	 ! temperatura do inflow
!--------------------------------------------------------------------------------------------------------------------
! Variaveis de controle de leitura de dados
integer:: Area_controle					! Se Area_controle=1 (considera area nos calculos); se Area_controle=0 (A=1)
integer:: Nivel_controle				! Se Nivel_controle=1 (considera nivel nos calculos); se Nivel_controle=0 
integer:: Vento_controle				! Se Vento_controle=1 (considera media movel do vento nos calculos); se Vento_controle=0 
integer:: Vazao_controle				! Se Vazao_controle=1 (considera vazao nos calculos); 
integer:: Temperatura_Vazao_controle	! Se Temperatura_Vazao_controle=1 (modelo estima Tin); 
integer:: Difusao_vazao_controle		! Se for para aumentar a difuso=1 (modelo estima um incremento de difuso com base na vazo de entrada); 
integer:: Distribuicao_vazao_controle	! definio de como ser distribuida a vazo. 1: em toda a profundidade; 0: acima da densidade proporcional a do rio
integer:: Termoclina_controle			! Se Termoclina_controle=1 o modelo calcula a termoclina
integer:: Epi_hipo_controle				! Se Epi_hipo_controle=1 o modelo calcula a espessura; profundidade do epi e hipolimnio
integer:: Wedderburn_controle			! Se Wedderburn_controle=1 o modelo calcula o numero de Wedderburn
integer:: Lake_controle					! Se Lake_controle=1 o modelo calcula o lake number
integer:: Brunt_controle				! Se Brunt_controle=1 o modelo calcula a frequencia de brunt vaisala
integer:: Froude_controle				! Se Froude_controle=1, o modelo calcula o numero de Froude
integer:: dT_controle					! Se dT_controle=1 o modelo calcula o dT superficie e fundo
!---------------------------------------------------------------------------------------------------------------------
  call cpu_time(time1)            ! tempo de processamento
  !---------------------------------------------------------------------------------------------------------------------
  ! Definio dos parmetros que sero usados no modelo
  open(unit=11, file='parametros.txt') 
  read(11,*)
  read(11,*)Area_controle                 ! definio se a area  considerada ou no
  read(11,*)Nivel_controle                ! definio se o nivel  considerado ou no
  read(11,*)Vazao_controle                ! definio se a vazao  considerada ou no
  read(11,*)Temperatura_Vazao_controle    ! definio se a temperatura do inflow  estimada pelo modelo ou no
  read(11,*)Vento_controle                ! definio se a mdia movel do vento  considerada ou no
  read(11,*)Difusao_vazao_controle        ! definio se o aumento da difuso pela vazo  considerada ou no
  read(11,*)Distribuicao_vazao_controle   ! definio de como ser distribuida a vazo. 1: em toda a profundidade; 0: acima da densidade proporcional a do rio
  
  close(11)
  !-------------------------------------------------------------------------------------------------------------------------
  ! Definio de quais indices fisicos sero calculados pelo modelo
  open(unit=12, file='definicao_indices_fisicos.txt')
  read(12,*)
  read(12,*)Termoclina_controle           ! definio se os ndices so calculados ou no
  read(12,*)Epi_hipo_controle
  read(12,*)Wedderburn_controle
  read(12,*)Lake_controle
  read(12,*)Brunt_controle
  read(12,*)Froude_controle
  read(12,*)dT_controle
  close(12)
  !-------------------------------------------------------------------------------------------------------------------------    
  ! Alocao para os passos de tempo   
  call delta_t(n)
  allocate(H(n+1))
  j=1           
  H(1)=Hmax
  ! Alocao de espao no primeiro passo de tempo
  call delta_z (j, n, H, m)
  !-------------------------------------------------------------------------------------------------------------------------    
  ! Alocao das matrizes, com mximo no espao para garantir que no haver erro
  allocate (T(m_max), Tn(m_max), rho(m_max), cp(m_max), Kh(m_max), q(m_max), Qin(m_max), Qout(m_max))
  allocate (UR(n+1), U0(n+1), U(n+1), Ta(n+1), qs(n+1), qn(n+1))
  allocate (dia (n+1), mes(n+1), ano(n+1), Q_in(n+1), Q_out(n+1), Tam(n+1), Tin(n+1))
  allocate (z(m_max), w(m_max), N2(m_max), Ri(m_max), Tz(m_max), E(n+1), BE(n), BE1(n))
  allocate (a(m_max),b(m_max),c(m_max),d(m_max))
  allocate (Hn(n+1), zh(m_max), nivel(n+1), Haux(m_max), Kh_vazao(m_max), rhoaux(m_max, n+1))
  allocate (Taux(m_max,n+1), maux(n+1), zaux(m_max,n+1), Ar(m_max),Khaux(m_max, n+1), Riaux(m_max, n+1), N2aux(m_max, n+1))
  !-------------------------------------------------------------------------------------------------------------------------    
  ! Leitura dos dados de entrada         
  !-------------------------------------------------------------------------------------------------------------------------    
  ! Leitura da condio inicial
  open (unit = 20, file = "condicao_inicial.txt")
  do i = m, 1, -1
      read (20, *) T(i)
  end do
  close(20)
  ! Leitura dos dados meteorologicos
  open (unit = 21, file = "dados_meteorologicos.txt")
  do j = 1, n
      ! Leitura dos dados meteorolgicos
      read (21, *) U0(j), Ta(j), UR(j), qs(j), dia(j), mes(j), ano(j)
  end do 
  close(21)
  ! Leitura da vazo
  open (unit = 22, file = "inflow_outflow.txt")
  do j = 1, n
      ! Leitura dos dados de vazo
      read (22, *) Q_in(j), Q_out(j)
  end do 
  close(22) 
    ! Leitura da area (opcional)
  open (unit = 33, file = 'area.txt')
  if (Area_controle==1)then
    do i= m_max, 1, -1    
      read (33, *) Ar (i)
    end do
    else
      do i= m_max, 1, -1
        Ar(i)=1.d0                                      
      end do
  end if      
  close(33)   
  ! Nvel do reservatrio                                                             
  if(Nivel_controle==1)then
    call nivel_medio (n, nivel)
    else
      do j=1, n
        nivel(j)=0
      end do  
  end if  
  ! Vazo no reservatrio
  if(Vazao_controle/=1)then
      do j=1, n
          Q_in(j)=0
          Q_out(j)=0
      end do  
  end if  
  ! Mdia mvel para o vento
  if(Vento_controle==1)then
    call vento (n, U0, U)     ! mdia mvel
    else
      do j=1, n
          U(j)=U0(j)
      end do
  end if  
  ! Temperatura da vazo que entra no reservatrio
  if (Temperatura_Vazao_controle==1)then
    call temperatura_rio(Ta, n, Tas, Tam, Tin)            
    else
      open(unit=100, file='temperatura_inflow.txt')                                                                   !talvez  bom colocar como leitura diaria =D
      do j=1, n
        read(100,*) Tin(j)                            
      end do
      close(100)
  end if    
  !-------------------------------------------------------------------------------------------------------------------------         
  ! Calculo de H1
  H(1)=Hmax-nivel(1)
  !-------------------------------------------------------------------------------------------------------------------------         
  do j = 1, n
        call delta_z (j, n, H, m)         ! calcula o numero de volumes em cada dt
        maux(1)=m
        
        if (U(j) < 0.5d0)then ! Restrio para a velocidade do vento (d problema sem essa condio)
           U(j)=0.5d0
        end if  
        
  		call difusao (H, U, m, T, j, n, rho, cp, N2, Ri, Kh) 
        call inflow (Distribuicao_vazao_controle, rho, Tin, Q_in, Q_out, m, n, j, Qin, Qout)			!!! deixar no mdulo clculos???
  
        if(Difusao_vazao_controle==1)then
          call difusao_vazao(m, j, n, H, Qin, Kh_vazao) 
          do i=2, m-1
            Kh(i)=Kh(i)+Kh_vazao(i)
          end do  
        end if    
        
        call fluxo_qn (maux, m, n, j, T, U, Ta, UR, qs, qn)       		! fluxos de calor na interface gua-ar
        call coeficientes(Ar, maux, H, m, n, j, qs, qn, Kh, T, rho, cp, a, b, c, d, &
                          & Qin, Qout, q, Tin)							! calcula os coeficientes do sistema de equaes
        call solver (m, a, b, c, d, Tn)           						! resolve o sistema de equaes tridiagonal
        call estabiliza (Ar, m, Tn, Tz)    								! verifica a superficie e corrige instabilidades
        call estabiliza_fundo (Ar, m, Tz, T)							! verificao de instabilidades
  
        ! Variveis auxiliares para calcular as medias e facilitar visualizaes
        do i=1, m
          Taux(i,j)=T(i)
          Khaux(i,j)=Kh(i)
          Riaux(i,j)=Ri(i)
          N2aux(i,j)=N2(i)
          rhoaux(i,j)=rho(i)
        end do
        maux(j+1)=m   
        ! Atualizao da profundidade do reservatrio
        H(j+1)=Hmax-nivel(j)
  end do
  !-------------------------------------------------------------------------------------------------------------------------         
  ! Resultados do modelo
  open (unit = 5, file = "info.txt")
  write (5, *)  H(1), "%H = "
  write (5, *)  m, "%m = "
  write (5, *)  dz, "%dz = " 
  write (5, *)  tempo, "%Tempo total = "
  write (5, *)  t_saida, "%Tempo para as mdias de temp simuladas = "
  write (5, *)  n, "%n = "
  write (5, *)  dt, "%dt = "
  write (5, *)  t_dados_Temp, "%Tempo de temp medidos = " 
  write (5, *)  Hmax, "%profundidade mxima no perodo"
  
     ! Para o perfil de temperatura mdio
     aux_media=0
     call media(aux_media, dia, mes, ano, maux, Taux, n, m_aux, Ts)    ! retorna os perfis de temperatura na escala de tempo desejada
     aux_media=3					! para os ndices fsicos
     call media(aux_media, dia, mes, ano, maux, rhoaux, n, m_aux, rhos)    ! retorna os perfis da frequencia de brunt vaisala na escala de tempo desejada
     ! Dados meteorologicos medios
     call saida_dados_meteorologicos(n, U0, qs, UR, Ta, Us, URs, qss, Tas)
  !-------------------------------------------------------------------------------------------------------------------------         
  ! Indices fsicos do modelo
      if (Termoclina_controle==1)then                 
        call termoclina (rhos, m_aux, aux)
      end if
      if (Epi_hipo_controle==1.and.Termoclina_controle==1)then
        call hipo_epi_depth (m_aux,aux, Ts, aux_ep_depth, aux_hipo_depth, epilimnio, hipolimnio, rho_hipo, rho_epi)
      end if
      if (Wedderburn_controle==1.and.Epi_hipo_controle==1.and.Termoclina_controle==1)then
        call Wedderburn (Us, epilimnio, rho_hipo, rho_epi)
      end if
      if(Lake_controle==1.and.Epi_hipo_controle==1.and.Termoclina_controle==1)then
        call lake_number (m_aux, Ar, Us, H, epilimnio, hipolimnio, rho_hipo)
      end if  
      if(Brunt_controle==1)then
        call brunt_vaisala(m_aux, rhos)
      end if
      if(Froude_controle==1)then
        call froude(m_aux, Tin, Q_in, Q_out, rhos)
      end if
      if(dT_controle==1)then
        call dT_perfis(m_aux,Ts)
      end if 
  !-------------------------------------------------------------------------------------------------------------------------          
  ! Calculo do tempo de processamento do modelo
  call cpu_time(time2)
  time3=time2-time1
  write(*,*)"Tempo de processamento do modelo = ", time3, ' s'
  !-------------------------------------------------------------------------------------------------------------------------         
  stop
end program


