module estabilizacao
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Modulo para estabilizao dos perfis de temperatura
!
contains
!
!--------------------------------------------------------------------------------
! estabiliza: estabilizao dos perfis de temperatura
!			
! Varivel de interesse: Temperatura
!
subroutine estabiliza (Ar, m, Tn, Tz)
!
use constantes
implicit none
! 
integer:: m
integer:: i	
integer::f										! Contador no espao
double precision, dimension(m):: Tn				! Temperatura da gua
double precision, dimension(m):: Tz				! Temperatura da gua
double precision, dimension (m) :: Ar			! vetor rea
double precision:: soma
double precision:: soma_area

! Estabilizao do perfil de temperatura
do i=1, m
    Tz(i)=Tn(i)
end do

do i= m-1, 3, -1
   if (Tz(i) < Tz(i-1).or.Tz(i-1) < Tz(i-2))then
     soma=0.d0
     soma_area=0.d0
     do f=m-1, i-1, -1
          soma=soma+Tz(f)*Ar(f)
          soma_area=soma_area+Ar(f)
     end do
    soma = soma/soma_area
    do f= m-1, i-1, -1
          Tz(f)=soma
     end do
     else
       exit
  end if
end do

return
end subroutine
!--------------------------------------------------------------------------------
! estabiliza: estabilizao dos perfis de temperatura, verificao do fundo por causa da vazo
!			
! Varivel de interesse: Temperatura
!
subroutine estabiliza_fundo (Ar, m, Tn, Tz)
!
use constantes

!
implicit none
! 
integer:: m
integer:: i										! Contador no espao
integer::f
double precision, dimension(m):: Tn				! Temperatura da gua de entrada
double precision, dimension(m):: Tz				! Temperatura da gua estabilizada
double precision:: soma
double precision, dimension (m) :: Ar
double precision:: soma_area

! Estabilizao do perfil de temperatura
do i=1, m
    Tz(i)=Tn(i)
end do
    
do i= 2, m-2 
   if (Tz(i+1) < Tz(i).or.Tz(i+2) < Tz(i+1))then
     soma=0.d0
     soma_area=0.d0
     do f=2, i+1
          soma=soma+Tz(f)*Ar(f)
          soma_area=soma_area+Ar(f)
     end do
    soma = soma/soma_area
     do f= 1, i+1
          Tz(f)=soma
     end do
     else
       exit
  end if
end do

return
end subroutine
!----------------------------------------------------------------------
end module estabilizacao


