% Indices fisicos calculados pelo modelo de temperatura 

clear all
close all

load lake_number.txt
load Wedderburn.txt
load termoclina.txt
load depth.txt
load perfis_medios.txt
load info.txt
load Brunt_Vaisala.txt
load dTemp.txt
load froude.txt

Ln=lake_number(:,3);
W=Wedderburn(:,2);
z_term=termoclina(:,2);     
z = depth(:,1);    
T1 = perfis_medios (:,:); 
T1(find(T1== -9.999000000000000e+02)) = NaN;
T=T1';
N2=Brunt_Vaisala(:,2);
dT=dTemp(:,2);
fr_in=froude(:,2);
fr_out=froude(:,3);

t_simulacao = info(4,1);   % tempo total de simulao
t_saida = info(5,1);       % tempo em que so gerados os perfis : horrios, etc...
n =t_simulacao / t_saida;  % nmero de perfis gerados pela mdia horria
j = 0 : n-1;
k=30;                       %intervalo em que a data vai ser impressa
lim=1.0;                    % valor critico de W e Ln

% read text file (l a data comouma string)
fid = fopen('time.txt','rt'); 
C = textscan(fid,'%s ');  % reads string string string number
fclose(fid); % closes file

%associar parametros as colunas
time = C{1};  % combines date and time into one string
tempo = time;%datenum(time,'dd/mm/yyyy');

figure(1)
contourf (j, z, T, 10);
[CC,h] = contourf(j,z,T);
for q=1:length(h)
set(h(q),'LineStyle','none');
end
%xlabel('dias');
Xt = 0:k:n-1;
Xl = [0 n-1];
set(gca,'XTick',Xt,'XLim',Xl);
ax = axis; 
axis(axis); 
Yl = ax(3:4); 
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels
ylabel('Elevao (m)');
caxis([25 30]);
colorbar;              
shading flat; 
set(gcf,'Color','w'); 
legend('T(C)')
hold on
    plot(j,z_term,'w')
hold off


figure(2)
[AX,H1,H2] = plotyy(j, W, j, Ln, 'semilogy');
box on;
set(get(AX(1),'Ylabel'),'String','W') 
set(get(AX(2),'Ylabel'),'String','Ln')
%xlabel('dias');
Xt = 1:k:n-1;
Xl = [1 n-1];
set(AX(1), 'xlim', [0 n])
set(AX(2), 'xlim', [0 n])
ax = axis; 
axis(axis);
Yl = ax(3:4); 
set(gcf,'Color','w');           
legend('Nmero de Wedderburn','Nmero do Lago', 'Location', 'Northwest')
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels
hold on
 plot(j, lim)
hold off 

figure(3)
plot(j,N2, 'LineWidth', 1.5)
%xlabel('dias');
ylabel('N^2 (s^{-2})');
Xt = 0:k:n-1;
Xl = [0 n-1];
set(gca,'XTick',Xt,'XLim',Xl);
ax = axis; 
axis(axis); 
Yl = ax(3:4);
set(gcf,'Color','w');           
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k','EdgeColor','k');
legend('Frequncia de Brnt-Visl', 'Location', 'Northwest')
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels

figure (4)
[AX,H1,H2] = plotyy(j, fr_in, j, fr_out);
box on;
set(get(AX(1),'Ylabel'),'String','Fr_{in}^2') 
set(get(AX(2),'Ylabel'),'String','Fr_{out}')
%xlabel('dias');
Xt = 0:k:n-1;
Xl = [0 n-1];
set(AX(1), 'xlim', [0 n])
set(AX(2), 'xlim', [0 n])
ax = axis; 
axis(axis);
Yl = ax(3:4); 
set(gcf,'Color','w');          
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k','EdgeColor','k');
legend('Nmero de Froude', 'Location', 'Northwest')
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels

figure(5)
plot(j,dT, 'LineWidth', 1.5)
ylabel('\Delta T (C)');
%xlabel('dias');
Xt = 0:k:n-1;
Xl = [0 n-1];
set(gca,'XTick',Xt,'XLim',Xl);
ax = axis; 
axis(axis);
Yl = ax(3:4); 
set(gcf,'Color','w');          
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k','EdgeColor','k');
legend('Modelo','Location', 'Northwest')
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels

figure(6)
plot(j,T(47,:), 'LineWidth', 1.5)
ylabel('Temperatura na camada superficial (C)');
%xlabel('dias');
Xt = 0:k:n-1;
Xl = [0 n-1];
set(gca,'XTick',Xt,'XLim',Xl);
ax = axis; 
axis(axis);
Yl = ax(3:4); 
set(gcf,'Color','w');          
h = findobj(gca,'Type','patch');
set(h,'FaceColor','k','EdgeColor','k');
legend('Modelo','Location', 'Northwest')
% Place the text labels
t = text(Xt,Yl(1)*ones(1,length(Xt)),time(1:k:n-1,:)); 
set(t,'HorizontalAlignment','right','VerticalAlignment','top', ...
'Rotation',45);
set(gca,'XTickLabel','')
% end text labels

mean (T(47,:))


