module indices_fisicos
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Mdulo que calcula os ndices fsicos
!
contains
!----------------------------------------------------------------------
! calcula a termoclina
subroutine termoclina (rhos, m_aux, aux)

use constantes

implicit none

integer:: j
integer:: i
integer:: k
integer, dimension(np):: m_aux		! auxiliar para calcular os indices fisicos
double precision, dimension (m_max, np):: rhos     ! Massa especfica da gua
double precision, dimension (np):: z_termoclina  ! termoclina, usando a relao do lake analyzer


double precision, dimension (m_max, np) :: aux4
double precision:: rtr
integer, dimension(np):: aux
double precision:: rho4
double precision:: rho5
double precision:: aux_plot
	
open(unit=145, file='termoclina.txt')
open(unit=144, file='RTR.txt')
open(unit=150, file='massa_especifica_media.txt')
do j=1, np
  read(150,*)(rhos(i,j), i=1, m_aux(j))
end do
close(150)
!-----------------------------------------------------------------
aux_plot=-999.9
!Auxiliares do metodo RTR
       rho4 = (999.842594d0 + 0.06793952d0 * 4.d0 - 0.009095290d0 * 4.d0 ** 2 &
                     + 0.0001001685d0 * 4.d0 ** 3 - 0.000001120083d0 * 4.d0 ** 4 &
                     + 0.000000006536332d0 * 4.d0 ** 5)
       rho5 = (999.842594d0 + 0.06793952d0 * 5.d0 - 0.009095290d0 * 5.d0 ** 2 &
                    + 0.0001001685d0 * 5.d0 ** 3 - 0.000001120083d0 * 5.d0 ** 4 &
                    + 0.000000006536332d0 * 5.d0 ** 5) 
rtr=0.d0
!----------------------------------------------------------------------------------------------
do j=1, np
     ! Mtodo: RTR: relative thermal resistance
        aux(j)=0
        do i=3, m_aux(j)-2
        aux4(i,j)=(rhos(i,j)-rhos(i-1,j))/(rho5-rho4)
        if(aux4(i,j)>rtr)then
            rtr=aux4(i,j)
            aux(j)=i
        end if
    end do
    z_termoclina(j)=((aux(j)*dz+dz/2)+((aux(j)-1)*dz+dz/2))/2.d0
    rtr=0.d0
    write(145,*)j, z_termoclina(j)
end do
!-----------------------------------------------
! RESULTADOS
101 format (100(E11.5, 1x))
do j=1, np      
    aux4(m_aux(j)-1,j)=0.d0
    write (144, 101)(aux4(i,j), i=3, m_aux(j)-1),(aux_plot, k= m_aux(j), m_max+1)          ! Plot da resistencia trmica relativa
end do
 
return
end subroutine termoclina
!-----------------------------------------------------------------------------------------
! calcula a termoclina
subroutine hipo_epi_depth (m_aux, aux, Ts, aux_ep_depth, aux_hipo_depth, epilimnio, hipolimnio, rho_hipo, rho_epi)!rhos, 

use constantes

implicit none

integer:: j
integer:: i
double precision:: drho
double precision::aux_ep							! variavel auxiliar
double precision::aux_hipo							! variavel auxiliar
integer, dimension(np):: m_aux						! auxiliar para calcular os indices fisicos
integer, dimension(np):: aux 						! variavel auxiliar                    
integer, dimension (np)::aux_ep_depth				! variavel auxiliar
integer, dimension (np)::aux_hipo_depth				! variavel auxiliar
double precision, dimension (m_max,np):: Ts       	! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (m_max, np):: rhos     	! Massa especfica da gua
double precision, dimension (np):: hipolimnio   	! profundidade do hipolimnio  
double precision, dimension (np):: epilimnio    	! profundidade do epilimnio
double precision, dimension (np):: rho_epi      	! densidade do epilimnio
double precision, dimension (np):: rho_hipo      	! densidade do hipolimnio

open(unit=150, file='hipo_epi_depth.txt')
open(unit=151, file='massa_especifica_media.txt')
!--------------------------------------------------------------------------------------------
do j=1, np
  read(151,*)(rhos(i,j), i=1, m_max-1)
end do
close(151)
!-------------------------------------------------------------------------------------------
! Profundidade do Epilimnio
do j=1, np
    aux_ep=0.01
    aux_ep_depth(j)=aux(j)
    rho_epi (j)=0.d0                  ! para criar controle!
    if(aux(j)>m_aux(j)-2)then
      aux(j)=m_aux(j)-2
    end if  
    do i=aux(j), m_aux(j)-2
      if (aux(j)==0)then
        epilimnio(j)=0.d0
        rho_epi (j) = (999.842594d0 + 0.06793952d0 * Ts(2,j) - 0.009095290d0 * Ts(2,j) ** 2 &
                         + 0.0001001685d0 * Ts(2,j) ** 3 - 0.000001120083d0 * Ts(2,j) ** 4 &
                         + 0.000000006536332d0 * Ts(2,j) ** 5)
        else
          drho=(rhos(i,j)-rhos(i+1,j))/dz
          if(drho<aux_ep)then
            aux_ep=drho
            aux_ep_depth(j)=i
            rho_epi (j) = (999.842594d0 + 0.06793952d0 * Ts(aux_ep_depth(j),j) - 0.009095290d0 * Ts(aux_ep_depth(j),j) ** 2 &
                         + 0.0001001685d0 * Ts(aux_ep_depth(j),j) ** 3 - 0.000001120083d0 * Ts(aux_ep_depth(j),j) ** 4 &
                         + 0.000000006536332d0 * Ts(aux_ep_depth(j),j) ** 5)
            exit
          end if  
          if(rho_epi (j)==0.d0)then
              aux_ep_depth(j)= m_aux(j)-1
              rho_epi(j) = (999.842594d0 + 0.06793952d0 * Ts(aux_ep_depth(j),j) - 0.009095290d0 * Ts(aux_ep_depth(j),j) ** 2 &
                         + 0.0001001685d0 * Ts(aux_ep_depth(j),j) ** 3 - 0.000001120083d0 * Ts(aux_ep_depth(j),j) ** 4 &
                         + 0.000000006536332d0 * Ts(aux_ep_depth(j),j) ** 5)
          end if
      end if
   end do
   if (rho_epi(j)==0.d0)then
       rho_epi (j) = (999.842594d0 + 0.06793952d0 * Ts(2,j) - 0.009095290d0 * Ts(2,j) ** 2 &
                         + 0.0001001685d0 * Ts(2,j) ** 3 - 0.000001120083d0 * Ts(2,j) ** 4 &
                         + 0.000000006536332d0 * Ts(2,j) ** 5)
   end if                
   epilimnio(j)=((aux_ep_depth(j)*dz+dz/2)+((aux_ep_depth(j)-1)*dz+dz/2))/2.d0
end do  
!---------------------------------------------------------------------------------------------
! Profundidade do Hipolimnio
do j=1, np
    aux_hipo=0.01
    aux_hipo_depth(j)=aux(j)
    rho_hipo (j)=0.d0                 ! para criar controle!
    do i=aux(j), 3, -1
      if (aux(j)==0)then
        hipolimnio(j)=0.d0
        rho_hipo (j) = (999.842594d0 + 0.06793952d0 * Ts(2,j) - 0.009095290d0 * Ts(2,j) ** 2 &
                         + 0.0001001685d0 * Ts(2,j) ** 3 - 0.000001120083d0 * Ts(2,j) ** 4 &
                         + 0.000000006536332d0 * Ts(2,j) ** 5)
                         
         else  
         drho=-(rhos(i,j)-rhos(i-1,j))/dz
         if(drho<aux_hipo)then
           aux_hipo=drho
           if(aux(j)<3)then
             aux(j)=2
           end if 
           aux_hipo_depth(j)=i
           rho_hipo (j) = (999.842594d0 + 0.06793952d0 * Ts(aux_hipo_depth(j),j) - 0.009095290d0 * Ts(aux_hipo_depth(j),j) ** 2 &
                        + 0.0001001685d0 * Ts(aux_hipo_depth(j),j) ** 3 - 0.000001120083d0 * Ts(aux_hipo_depth(j),j) ** 4 &
                        + 0.000000006536332d0 * Ts(aux_hipo_depth(j),j) ** 5)
           exit  
         end if 
         if(rho_hipo(j)==0.d0)then
           aux_hipo_depth(j)=2
           rho_hipo (j) = (999.842594d0 + 0.06793952d0 * Ts(aux_hipo_depth(j),j) - 0.009095290d0 * Ts(aux_hipo_depth(j),j) ** 2 &
                        + 0.0001001685d0 * Ts(aux_hipo_depth(j),j) ** 3 - 0.000001120083d0 * Ts(aux_hipo_depth(j),j) ** 4 &
                        + 0.000000006536332d0 * Ts(aux_hipo_depth(j),j) ** 5)
         end if 
        end if    
    end do
    if (rho_hipo(j)==0.d0)then
           rho_hipo (j) = (999.842594d0 + 0.06793952d0 * Ts(2,j) - 0.009095290d0 * Ts(2,j) ** 2 &
                         + 0.0001001685d0 * Ts(2,j) ** 3 - 0.000001120083d0 * Ts(2,j) ** 4 &
                         + 0.000000006536332d0 * Ts(2,j) ** 5)
     end if   
     hipolimnio(j)=((aux_hipo_depth(j)*dz+dz/2)+((aux_hipo_depth(j)-1)*dz+dz/2))/2.d0
     write(150,*)j, epilimnio(j), hipolimnio(j)
end do 
return
end subroutine hipo_epi_depth
!-----------------------------------------------------------------------------------------
! calcula o numero de Wedderburn
subroutine Wedderburn (Us, epilimnio, rho_hipo, rho_epi)

use constantes
implicit none

integer:: j
integer:: aux									! variavel auxiliar
double precision:: ws							! velocidade de corte
double precision, dimension (np):: rho_epi     	! Massa especfica da gua no epilimnio
double precision, dimension (np):: rho_hipo     ! Massa especfica da gua no hipolimnio
double precision, dimension (np):: Us			! velocidade media do vento
double precision, dimension (np):: epilimnio	! profundidade do epilimnio
double precision, dimension (np):: g_red		! gravidade reduzida
double precision, dimension (np):: W			! nmero de Wedderburn

! Arquivo de sada
open(unit=150, file='Wedderburn.txt')		! nmero de Wedderburn
open (unit=153, file='W_maior_1.txt')		! nmero de dias que W>1

aux=0
!----------------------------------------------------------------- 
! Densidade da gua no epilimnio e hipolimnio
do j=1, np
      g_red(j)=abs((rho_hipo(j)-rho_epi(j))/rho_hipo(j))
      ws = 0.0012d0 * Us(j)
      W(j)=g_red(j)*epilimnio(j)**2/(ws**2*Lf)
      write(150, *)j, W(j)
      if(W(j)>1.d0)then
      	aux=aux+1
      end if  
end do  
write(153,*)aux, '% n dias que W>1'
  
return
end subroutine Wedderburn
!-----------------------------------------------------------------------------------------
! calcula o numero do lago
subroutine lake_number (m_aux, A, Us, H, epilimnio, hipolimnio, rho_hipo)

use constantes
implicit none

integer:: j
integer:: i
integer:: aux									! auxiliar
double precision:: ws							! velocidade de corte
double precision:: zv							! centro de volume 
double precision:: soma_aux1					! auxiliar
double precision:: soma_aux2					! auxiliar
double precision:: soma_auxSt  					! auxiliar
integer, dimension(np):: m_aux					! auxiliar para calcular os indices fisicos
double precision, dimension(m_max):: A			! Area
double precision, dimension (np):: rho_hipo     ! Massa especfica da gua no hipolimnio
double precision, dimension (np):: Us			! velocidade media do vento
double precision, dimension(np):: H				! Profundidade do reservatrio 
double precision, dimension (np):: hipolimnio   ! profundidade do hipolimnio  
double precision, dimension (np):: epilimnio    ! profundidade do epilimnio
double precision, dimension(np):: St			! Estabilidade de Schimidt
double precision, dimension(np):: Ln			! Lake number
double precision, dimension (m_max,np):: rhos	! massa especifica da gua com escala de tempo diferente da calculada, e maior que dt

open(unit=151, file='Lake_number.txt')
open (unit=154, file='Ln_maior_1.txt')
open (unit=150, file='massa_especifica_media.txt')
!-----------------------------------------------------------------
do j=1, np
  read(150,*)(rhos(i,j), i=1, m_max-1)
end do
close(150)
!-----------------------------------------------------------------
! Clculo do centro de volume da gua - zv
aux=0
do j=1, np
	 soma_aux1=0.d0					! auxiliar para zv	
	 soma_aux2=0.d0					! auxiliar para zv
     soma_auxSt=0.d0				! auxiliar da estabilidade de Schimidt
     do i=2, m_aux(j)-1
       soma_aux1=soma_aux1+(H(j)-(i-1)*dz+dz/2)*A(i)*dz
       soma_aux2=soma_aux2+A(i)*dz
     end do  
     zv=soma_aux1/soma_aux2
	 ! Estabilidade de Schimidt
     do i=2, m_aux(j)-2
        soma_auxSt=soma_auxSt+(g/A(m_aux(j)-1))*((H(j)-(i-1)*dz+dz/2-zv)*rhos(i,j)*A(i)*dz) 
     end do
     St(j)= soma_auxSt
     ! Lake number
     ws = 0.0012d0 * Us(j)
     Ln(j)=St(j)*((epilimnio(j)-hipolimnio(j)))/(2.d0*rho_hipo(j)*ws**2*A(m_aux(j)-1)**(1./2.)*zv)
     write(151,*)j, St(j), Ln(j)
     if(Ln(j)>1.d0)then
       aux=aux+1
     end if  
end do
write(154,*)aux, '% n dias que Ln>1'     
return
end subroutine lake_number
!-----------------------------------------------------------------------------------------
! Subrotina para calcular a frequencia de Brunt-Vaisala
subroutine brunt_vaisala(m_aux, rhos)

use constantes
implicit none

integer:: j
integer, dimension(np):: m_aux						! auxiliar para calcular os indices fisicos
double precision:: rho_super						! massa especifica na superfcie
double precision:: rho_fundo						! massa especifica no fundo
double precision, dimension (m_max,np):: rhos		! massa especifica mdia
double precision, dimension (np):: N2				! frequencia de Brunt Vaisala

open(unit=11,file='Brunt_Vaisala.txt')

do j=1, np
   rho_super  = rhos(m_aux(j)-10,j)
   rho_fundo=rhos(10,j)
   if((rho_super-rho_fundo==0.d0))then
     N2(j)=0.d0
   	 else  
   		N2(j)=-g/1000.d0*(rho_super-rho_fundo)/((m_aux(j)-1)*dz+dz/2-dz/2)     
   end if              
   write(11,*)j, N2(j)                 
end do  
return
end subroutine brunt_vaisala
!--------------------------------------------------------------------------------------------
! subrotina para calcular dT=Tsuperficie-Tfundo
subroutine dT_perfis(m_aux, Ts)

use constantes

implicit none

integer:: j
integer:: aux1									! auxiliar
integer:: aux2									! auxiliar
integer:: aux3									! auxiliar
integer, dimension(np):: m_aux					! auxiliar para calcular os indices fisicos
double precision, dimension (m_max,np):: Ts	    ! temperatura da gua com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: dTemp		! diferena de temperatura (superficie e fundo)

open(unit=55, file='dTemp.txt')
open(unit=56, file='dias_dt.txt')
aux1=0
aux2=0
aux3=0
do j=1, np
	dTemp(j)=Ts(m_aux(j)-10,j)-Ts(10,j)
    if(dTemp(j)>1.d0)then
      aux1=aux1+1
    end if  
    if(dTemp(j)>2.d0)then
      aux2=aux2+1
    end if  
    if(dTemp(j)>3.d0)then
      aux3=aux3+1
    end if  
	write(55,*)j, dTemp(j)
end do  
write(56,*)aux1, '% dias que dt > 1C'
write(56,*)aux2, '% dias que dt > 2C'
write(56,*)aux3, '% dias que dt > 3C'

return
end subroutine dT_perfis
!--------------------------------------------------------------------------------------------
! subrotina para calcular n de froude
subroutine froude(m_aux, Tin, Q_in, Q_out, rhos)

use constantes

implicit none

integer:: j
double precision:: glinhai						! gravidade reduzida entrada
double precision:: glinhao						! gravidade reduzida saida
integer, dimension(np):: m_aux					! auxiliar para calcular os indices fisicos
double precision, dimension(np):: Q_out			! vazo saida
double precision, dimension(np):: Q_in			! vazo entrada
double precision, dimension(np):: rho_in		! massa especifica da Q_in
double precision, dimension(np):: Tin			! temperatura da Tin
double precision, dimension(np):: Fri			! Froude inflow
double precision, dimension(np):: Fro			! Froude outflow
double precision, dimension (m_max,np):: rhos	! densidade da gua com escala de tempo diferente da calculada, e maior que dt

open(unit=56, file='froude.txt')

do j=1, np
  rho_in(j) = (999.842594d0 + 0.06793952d0 * Tin(j) - 0.009095290d0 * Tin(j) ** 2 &
                      + 0.0001001685d0 * Tin(j) ** 3 - 0.000001120083d0 * Tin(j) ** 4 &
                      + 0.000000006536332d0 * Tin(j) ** 5) 
end do    
                 
do j=1, np
  ! Na entrada
	glinhai=g*(rhos(m_aux(j)-10,j)-rho_in(j))/rhos(m_aux(j)-10,j)
  	Fri(j)=Q_in(j)**2/(glinhai*(hf)**(3.d0)*(Bf)**(2.d0))			! ISSO  Fr**2
  ! Na sada
    glinhao=g*(rhos(10,j)-rhos(m_aux(j)-10,j))/rhos(10,j)
	if(glinhao==0.d0)then
    	Fro(j)=-999.99
    	else
      		Fro(j)=Q_out(j)/(glinhao**(1./2.)*(hf)**(5./2.))			! ISSO  Fr
  	end if    
    write(56,*)j, Fri(j), Fro(j)
end do  

return
end subroutine froude
end module indices_fisicos



