module inflow_outflow
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Modulo para o distribuir a vazo na coluna d'gua
!
contains
!
!------------------------------------------------------------------
! Calculos referentes ao inflow e outflow do reservatrio
subroutine inflow (Distribuicao_vazao_controle, rho, Tin, Q_in, Q_out, m, n, j, Qin, Qout)
!
	use constantes

implicit none
! 

integer:: i 											! contador
integer:: j
integer:: n
integer:: m 
integer:: cont
integer:: Distribuicao_vazao_controle
double precision:: rho_in						! massa especifica inflow
double precision,  dimension (m):: rho			! massa especfica da gua
double precision, dimension (m):: Qin			! vazo de entrada em cada volume
double precision, dimension (m):: Qout			! vazo de sada em cada volume
double precision, dimension(n+1):: Q_in			! srie temporal vazo entrada
double precision, dimension(n+1):: Q_out		! srie temporal vazo sada
double precision, dimension(n+1):: Tin			! temperatura inflow


rho_in = (999.842594d0 + 0.06793952d0 * Tin(j) - 0.009095290d0 * Tin(j) ** 2 &
                      + 0.0001001685d0 * Tin(j) ** 3 - 0.000001120083d0 * Tin(j) ** 4 &
                      + 0.000000006536332d0 * Tin(j) ** 5) 

! Comparao da densidade do inflow e da coluna d'gua
cont=0		! auxiliar
do i=1, m
  Qin(i)=0.d0
  Qout(i)=0.d0
end do  
! Se for maior que a massa especifica do primeiro volume
if (rho_in > rho(2))then
  Qin(2)=Q_in(j)
  Qout(2)=Q_out(j)
  ! Se for menor que do volume mais prximo  superficie
  else if (rho_in < rho(m-1))then
    Qin(m-1)=Q_in(j)
    Qout(m-1)=Q_out(j)
    else 
    do i=m-2, 2, -1
        if(rho_in < rho(i))then
            exit
        else
            cont=cont+1    
        end if
    end do
end if    
! Distribuio da vazo
    ! Distribuio da vazo para cima, se Distribuicao_vazao_controle=0
    if (Distribuicao_vazao_controle==0)then
      do i=m-2-cont, m-1
        Qin(i)=Q_in(j)/(m-2-cont)
        Qout(i)=Q_out(j)/(m-2-cont)
	  end do
      ! Distribuio da vazo em toda a profundidade, se Distribuicao_vazao_controle=0
	  else	if (Distribuicao_vazao_controle==1)then
      	do i=2, m-1
    		Qin(i)=Q_in(j)/(m-2)
    		Qout(i)=Q_out(j)/(m-2)
		end do 
    end if    

return
end subroutine inflow
end module inflow_outflow