module resultados
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Mdulo para gerar dados para visualizao dos resultados
!
contains
!
!----------------------------------------------------------------------------------
! Subrotina para gerar perfis de temperatura em diferentes escalas de tempo
subroutine media(aux_media, dia, mes, ano, maux, Taux, n, m_aux, Ts)

use constantes

implicit none

integer:: n													! pontos em relao ao tempo
integer:: i													! contador do espao
integer:: j													! contador do tempo (considerando o dt do programa)
integer:: k													! contador do tempo para visualizao em escala de tempo diferente
integer:: l													! auxiliar nos clculos	
integer:: aux_media											! auxiliar
double precision:: aux										! auxiliar
integer, dimension(np):: m_aux								! auxiliar 
integer, dimension(n+1):: maux								! auxiliar
double precision, dimension (m_max,n+1):: Taux				! temperatura da gua
double precision, dimension (m_max,np):: Ts					! temperatura da gua com escala de tempo diferente da calculada, e maior que dt	
integer, dimension (:):: dia
integer, dimension (:):: mes
integer, dimension (:):: ano
double precision, allocatable, dimension (:):: soma1			! auxiliar na acumulao da temperatura

1010 format (90(E11.5, 1x))
allocate (soma1(m_max))

if(aux_media==0)then
  open(unit=100,file="perfis_medios.txt")
end if
if(aux_media==3)then
  open(unit=130,file="massa_especifica_media.txt")
end if  
! Rotina para calcular a mdia dos dados simulados: TEMPERATURA e massa especfica
l=1
do k=1, np
  m_aux(k)=maux(int((t_saida/dt)*(l)))
  l=l+1
end do

  do j=1, t_saida/dt
    do i=1,m_max
      soma1(i)=0.d0
    end do
  end do        
  do j=1, t_saida/dt
      do i=2, maux(int(t_saida/dt))-1
          ! Primeiro conjunto de pontos
          soma1(i)=soma1(i)+Taux(i,j)
      end do
  end do

   do i=2, maux(int(t_saida/dt))-1
       Ts(i,1)=soma1(i)/(t_saida/dt)
   end do    
   aux=-999.9
   if(aux_media==0)then
   		Ts(maux(int(t_saida/dt)),1)=Ts(maux(int(t_saida/dt))-1,1)
        write (100, 1010)(Ts(i,1), i=2,  maux(int(t_saida/dt))),& 
        &(aux, i= maux(int(t_saida/dt))+1, m_max)
   end if
   if(aux_media==3)then
        Ts(maux(int(t_saida/dt)),1)=Ts(maux(int(t_saida/dt))-1,1)
        write (130, 1010)(Ts(i,1), i=2,  maux(int(t_saida/dt))),& 
        &(aux, i= maux(int(t_saida/dt))+1, m_max)
   end if
! Inicializao de inteiros auxiliares nos clculos
l=2
k=2
do k=2, np
   do j=1, n
     do i=1,  m_max
       soma1(i)=0.d0
     end do
   end do 
   do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)
        do i=2, maux(j)-1
           soma1(i)=soma1(i)+Taux(i,j)                 
        end do
   end do
   do i=2, maux(j)-1
      Ts(i,k)=soma1(i)/(t_saida/dt) 
   end do
   if(aux_media==0)then
       Ts(maux(int((t_saida/dt)*(l))),k)=Ts(maux(int((t_saida/dt)*(l)))-1,k)
       write (100, 1010)((Ts(i,k)), i=2,  maux(int((t_saida/dt)*(l)))),&
       &(aux, i=1+maux(int((t_saida/dt)*(l))), m_max)
   end if
    if(aux_media==3)then
        Ts(maux(int((t_saida/dt)*(l))),k)=Ts(maux(int((t_saida/dt)*(l)))-1,k)
        write (130, 1010)((Ts(i,k)), i=2,  maux(int((t_saida/dt)*(l)))),&
        &(aux, i=1+maux(int((t_saida/dt)*(l))), m_max)
    end if
    l=l+1  
end do 
close(100)
close(130)

151 format(i2, A1, i2, A1, i4)
if(aux_media==0)then
   open (unit=150, file='time.txt')
   open (unit=160, file='depth.txt')
   do j=1, n, t_saida/dt
         write (150,151)dia(j), '/', mes(j),'/', ano(j)
      end do  
	  ! Profundidade
      do i=2,m_max
          write (160, *) ((i - 2) * dz + dz / 2.d0)
      end do 
      close(150)
      close(160)
end if
return
end subroutine media
!----------------------------------------------------------------------------------------
! Subrotina para gerar perfis de temperatura em diferentes escalas de tempo
subroutine saida_dados_meteorologicos(n, U, qs, UR, Ta, Us, URs, qss, Tas)

use constantes

implicit none

integer:: n
integer:: j													! contador do tempo (considerando o dt do programa)
integer:: k													! contador do tempo para visualizao em escala de tempo diferente
integer:: l													! auxiliar nos clculos	
double precision, dimension (n+1):: U					   
double precision, dimension (n+1):: qs					    
double precision, dimension (n+1):: UR					   
double precision, dimension (n+1):: Ta					   
double precision, dimension (np):: Us			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: qss			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: URs			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision, dimension (np):: Tas			! variavel desejada com escala de tempo diferente da calculada, e maior que dt
double precision:: soma							! auxiliar na acumulao 

open(unit=77, file='saida_dados_meteorologicos.txt')

! Rotina para calcular a mdia do Vento
    ! Primeiro conjunto de pontos
  	  soma=0.d0
      do j=1, t_saida/dt
           soma=soma+U(j)
      end do
      Us(1)=soma/(t_saida/dt) 
   ! Inicializao de inteiros auxiliares nos clculos
   l=1
   k=2
   do k=2, np
     	 soma=0.d0
         do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)   
           soma=soma+ U(j)   
         end do
         Us(k)=soma/(t_saida/dt)
         l=l+1   
   end do
   ! Rotina para calcular a mdia da radiao
    ! Primeiro conjunto de pontos
  	  soma=0.d0
      do j=1, t_saida/dt
           soma=soma+qs(j)

      end do
      qss(1)=soma/(t_saida/dt) 
   ! Inicializao de inteiros auxiliares nos clculos
   l=1
   k=2
   do k=2, np
     	 soma=0.d0
         do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)   
           soma=soma+ qs(j)   
         end do
         qss(k)=soma/(t_saida/dt)
         l=l+1   
   end do
  ! Rotina para calcular a mdia da UR
    ! Primeiro conjunto de pontos
  	  soma=0.d0
      do j=1, t_saida/dt
           soma=soma+UR(j)

      end do
      URs(1)=soma/(t_saida/dt) 
   ! Inicializao de inteiros auxiliares nos clculos
   l=1
   k=2
   do k=2, np
     	 soma=0.d0
         do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)   
           soma=soma+ UR(j)   
         end do
         URs(k)=soma/(t_saida/dt)
         l=l+1   
   end do
  ! Rotina para calcular a mdia da Ta
    ! Primeiro conjunto de pontos
  	  soma=0.d0
      do j=1, t_saida/dt
           soma=soma+Ta(j)

      end do
      Tas(1)=soma/(t_saida/dt) 
   ! Inicializao de inteiros auxiliares nos clculos
   l=1
   k=2
   do k=2, np
     	 soma=0.d0
         do j=(t_saida/dt)*(l-1)+1, (t_saida/dt)*(l)   
           soma=soma+ Ta(j)   
         end do
         Tas(k)=soma/(t_saida/dt)
         l=l+1   
   end do


! Visualizao dos dados   
	do k=1, np
       write(77,*)Us(k), Tas(k), URs(k), qss(k)
    end do   
  
return
end subroutine saida_dados_meteorologicos

end module resultados
 
    
