module solucao
!
! Modelo de Transporte de Calor unidimensional em reservatrios
! Implementao: Mtodo de volumes finitos
! Bruna Arcie Polli
! Orientador: Tobias Bleninger
! Mestrado em Engenharia de Recursos Hdricos e Ambiental - PPGERHA, UFPR 2014
! Modulo para o clculo dos perfis de temperatura
!
contains
!
!------------------------------------------------------------------
! coeficientes: Subrotina que calcula os coeficientes do sistema de equaes:
!				ap Tp = aw Tw + ae Te + bp
subroutine coeficientes(Ar, maux, H, m, n, j, qs, qn, Kh, T, rho, cp, a, b, c, d,  &
						& Qin, Qout, q, Tin)
!
	use constantes
    use inflow_outflow

implicit none
 
integer:: m
double precision, dimension(m):: a 				! Coeficiente ap do sistema de coeficentes
double precision, dimension(m):: b 				! Coeficiente aw do sistema de coeficentes
double precision, dimension(m):: c 				! Coeficiente ae do sistema de coeficentes
double precision, dimension(m):: d 				! Coeficiente b do sistema de coeficentes
double precision,  dimension (m):: rho			! massa especfica da gua
double precision,  dimension (m):: cp			! calor especifico da gua
double precision, dimension (m):: Kh			! difuso turbulenta
double precision, dimension (m):: q				! radiao solar -  lei de Beer, absorvida com a profundidade do lago
double precision, dimension (n+1):: qs			! radiao solar bruta
double precision, dimension(n+1):: qn			! Fluxo de energia na superfcie
double precision, dimension(n+1):: H			! Profundidade do reservatrio, tempo t
double precision, dimension (n+1):: Tin			! temperatura inflow
double precision, dimension(m):: Qin			! vazo entrada
double precision, dimension(m):: Qout			! vazo sada
double precision,  dimension (m):: T			! temperatura da gua
integer, dimension(n+1):: maux
double precision:: qsn							! radiao solar lquida								
integer:: i 										
integer:: j
integer:: n
integer:: k
integer:: kaux
double precision, dimension (m):: Ar

       

! Radiao absorvida com profundidade do lago

qsn=(1-as)*qs(j)
do i=1, m
  q(i)=((1 - beta) * qsn * dexp (- eta * (H(j) - (i - 1) * dz)))/(1.d0-dexp (- eta * (H(j)))) 
end do 
! Coeficientes do sistema de equaes discretizados 
! No contorno da esquerda i = 1
a(1) = 1.d0
b(1) = 0.d0
c(1) = 1.d0
d(1) = 0.d0
!-------------------------------------------
! Nos contornos interiores i=2,..., m+1
     kaux=maux(j)-m
do i= 2, m-1
    kaux=maux(j)-m
    if (kaux<=0)then
        do k=maux(j), m
            T(k)=T(maux(j)-1)
            cp(k)=cp(maux(j)-1)
            rho(k)=rho(maux(j)-1)
            
        end do  
   end if
   b(i) = Ar(i-1)*dt*Kh(i-1)/dz
   c(i) = Ar(i+1)*Kh(i+1)*dt/dz
   a(i) = Ar(i)*dz + b(i) + c(i) +(dz *dt *Ar(i)* Qout(i))/(Ar(i)*dz)							
   d(i) = Ar(i)*dz * T(i) + Ar(i)*dt/(rho(i)*cp(i))*(q(i+1)-q(i-1))+(Ar(i)*Qin(i)*Tin(j)*dz*dt)/(Ar(i)*dz)
!---------------------------------------------------
end do
! No contorno da direita i = m+2
a(m) = 1.d0
b(m) = 1.d0
c(m) = 0.d0
d(m) = ((qn(j)-(1-beta)*qsn)*dz)/(rho(m)*cp(m)*Kh(m))    
 
return
end subroutine coeficientes
!------------------------------------------------------------------
! solver: Subrotina que calcula o resultado do sistema de equaes:
!               ap Tp = aw Tw + ae Te + bp
! Varivel de interesse: Temperatura
!
subroutine solver (m, a, b, c, d, T)

    use constantes
    use calculos

implicit none

integer:: m
double precision, dimension(m):: a        ! Coeficiente ap do sistema de coeficentes
double precision, dimension(m):: b        ! Coeficiente aw do sistema de coeficentes
double precision, dimension(m):: c        ! Coeficiente ae do sistema de coeficentes
double precision, dimension(m):: d        ! Coeficiente b do sistema de coeficentes
double precision, dimension(m):: Qsol     ! Varivel auxiliar na soluo do sistema de equaes pelo mtodo TDMA
double precision, dimension(m):: P        ! Varivel auxiliar na soluo do sistema de equaes pelo mtodo TDMA
double precision, dimension(m):: T     	  ! Temperatura
integer:: i                                 ! contador

! Aplicao do mtodo TDMA
  Qsol(1) = d(1) / a(1)
  P(1) = c(1) / a(1)
    do i = 2, m-1 
        Qsol(i)=(b(i)*Qsol(i-1)+d(i))/(a(i)-b(i)*P(i-1))
        P(i)=c(i)/(a(i)-b(i)*P(i-1))  
    end do
    
    T(m)=(b(m)*Qsol(m-1)+d(m))/(a(m)-b(m)*P(m-1))
    do i=m-1, 1, -1
        T(i)=Qsol(i)+P(i)*T(i+1)
    end do

return
end subroutine solver
!----------------------------------------------------------------------
end module solucao  

